/***************************************************************************************
 * @file     va108xx_hal_debug.c
 * @version  V0.1
 * @date     13 February 2019
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2019 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdarg.h>
#include "va108xx.h"
#include "va108xx_debug.h"

#ifndef DEBUG_NO_HAL
#include "va108xx_hal_uart.h"
#endif

/*****************************************************************************/ 
/* Local pre-processor symbols/macros ('#define')                            */ 
/*****************************************************************************/

//#define DEBUG_BUFFER_LEN (128)

#ifdef DEBUG_NO_HAL
#define BaudToUARTCLKSCALE(baudRate) (((SYSCLK / ((baudRate) * 16)) << \
                           UART_PERIPHERAL_CLKSCALE_INT_Pos) |         \
                          (((((SYSCLK % ((baudRate) * 16)) *           \
                              64 + ((baudRate) * 8)) /                 \
                             ((baudRate) * 16))) <<                    \
                           UART_PERIPHERAL_CLKSCALE_FRAC_Pos))

#define UART0SendBreak(x) VOR_UART0->TXBREAK=(x)
#define UART1SendBreak(x) VOR_UART1->TXBREAK=(x)

#ifndef UARTA_BAUDRATE
#define UARTA_BAUDRATE (115200)
#endif
#ifndef UARTB_BAUDRATE
#define UARTB_BAUDRATE (115200)
#endif
#endif

/*****************************************************************************/ 
/* Global variable definitions (declared in header file with 'extern')       */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Local type definitions ('typedef')                                        */ 
/*****************************************************************************/

/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/

static en_stdio_t ioOut = en_stdio_none;

/*****************************************************************************/ 
/* Local function prototypes ('static')                                      */ 
/*****************************************************************************/

static void UART0Init(void);
static void UART1Init(void);
static void UART0TxB(uint8_t b);
static void UART1TxB(uint8_t b);

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

/*******************************************************************************
 **
 ** @brief Debug print
 **
 ******************************************************************************/
void DBG_Printf(const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  
  fprintf(stderr, "DBG: ");
  vfprintf(stderr, fmt, args);
  fflush(stderr);
  
  va_end(args);
}

/*******************************************************************************
 **
 ** @brief Debug print a line (puts newline at the end)
 **
 ******************************************************************************/
void DBG_Println(const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  
  fprintf(stderr, "DBG: ");
  vfprintf(stderr, fmt, args);
  fprintf(stderr, "\r\n");
  fflush(stderr);
  
  va_end(args);
}

/*******************************************************************************
 **
 ** @brief Configures UART0 peripheral.
 **
 ******************************************************************************/
static void UART0Init(void)
{
  /* Enable clock */
  VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_UARTA;

  /* Configure UART to operate at configUART_BAUDRATE. */
  VOR_UART0->CLKSCALE = BaudToUARTCLKSCALE(UARTA_BAUDRATE);
                   
  /* Configure word size (8 bits) and RTS behavior. */
  VOR_UART0->CTRL = (3 << UART_PERIPHERAL_CTRL_WORDSIZE_Pos) |
                    (UART_PERIPHERAL_CTRL_DEFRTS_Msk);

  /* Enable RX interupts as soon as a character is received. */
  //VOR_UART0->IRQ_ENB = UARTA_IRQ_ENB_IRQ_RX_Msk;
  //VOR_UART0->RXFIFOIRQTRG = 1;
  //VOR_IRQSEL->UART[0] = VOR_UART0_IRQ_INDEX;
  //NVIC_SetPriority(VOR_UART0_IRQ_INDEX, 1);
  //NVIC_EnableIRQ(VOR_UART0_IRQ_INDEX);	

  /* Enable UART. */
  VOR_UART0->ENABLE = (UART_PERIPHERAL_ENABLE_RXENABLE_Msk | 
                    UART_PERIPHERAL_ENABLE_TXENABLE_Msk);

  //send a break to let rx state machine reset										
  UART0SendBreak(32);
}

/*******************************************************************************
 **
 ** @brief Configures UART1 peripheral.
 **
 ******************************************************************************/
static void UART1Init(void)
{
  /* Enable clock */
  VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_UARTB;

  /* Configure UART to operate at configUART_BAUDRATE. */
  VOR_UART1->CLKSCALE = BaudToUARTCLKSCALE(UARTB_BAUDRATE);
                   
  /* Configure word size (8 bits) and RTS behavior. */
  VOR_UART1->CTRL = (3 << UART_PERIPHERAL_CTRL_WORDSIZE_Pos) |
                    (UART_PERIPHERAL_CTRL_DEFRTS_Msk);

  /* Enable RX interupts as soon as a character is received. */
  //VOR_UART1->IRQ_ENB = UARTA_IRQ_ENB_IRQ_RX_Msk;
  //VOR_UART1->RXFIFOIRQTRG = 1;
  //VOR_IRQSEL->UART[1] = VOR_UART1_IRQ_INDEX;
  //NVIC_SetPriority(VOR_UART1_IRQ_INDEX, 1);
  //NVIC_EnableIRQ(VOR_UART1_IRQ_INDEX);	

  /* Enable UART. */
  VOR_UART1->ENABLE = (UART_PERIPHERAL_ENABLE_RXENABLE_Msk | 
                    UART_PERIPHERAL_ENABLE_TXENABLE_Msk);

  //send a break to let rx state machine reset										
  UART1SendBreak(32);
}

/*******************************************************************************
 **
 ** @brief  Debug init / redirect
 **
 ******************************************************************************/
void DBG_SetStdioOutput(en_stdio_t io)
{
  ioOut = io;
#ifdef DEBUG_NO_HAL
  if(ioOut == en_stdio_uart0){
    UART0Init();
  }
  if(ioOut == en_stdio_uart1){
    UART1Init();
  }
#endif
}

/*******************************************************************************
 **
 ** @brief  TX a byte on UART0
 **
 ******************************************************************************/
static void UART0TxB(uint8_t b)
{
  // Block if UART TX FIFO is full
  while((VOR_UART0->TXSTATUS & UART_PERIPHERAL_TXSTATUS_WRRDY_Msk) == 0)
  {
  }
  
  // Send the byte
  VOR_UART0->DATA = b;
}

/*******************************************************************************
 **
 ** @brief  TX a byte on UART1
 **
 ******************************************************************************/
static void UART1TxB(uint8_t b)
{
  // Block if UART TX FIFO is full
  while((VOR_UART1->TXSTATUS & UART_PERIPHERAL_TXSTATUS_WRRDY_Msk) == 0)
  {
  }
  
  // Send the byte
  VOR_UART1->DATA = b;
}

/*******************************************************************************
 **
 ** @brief  Overrides standard fputc
 **
 ******************************************************************************/
int fputc(int c, FILE *fPointer)
{
  uint32_t dm;
  switch(ioOut)
  {
    case en_stdio_none:
      break;
    case en_stdio_uart0:
#ifndef DEBUG_NO_HAL
      HAL_Uart_TxByte(VOR_UART0, (uint8_t)c);
#else
      UART0TxB((uint8_t)c);
#endif
      break;
    case en_stdio_uart1:
#ifndef DEBUG_NO_HAL
      HAL_Uart_TxByte(VOR_UART1, (uint8_t)c);
#else
	    UART1TxB((uint8_t)c);
#endif
      break;
    case en_stdio_porta:
      dm = VOR_GPIO->BANK[0].DATAMASK;
      VOR_GPIO->BANK[0].DATAMASK  = 0xff;       // Write DataMask
      VOR_GPIO->BANK[0].DATAOUT   = 0x80 | c;  // Pulse bit 7
      VOR_GPIO->BANK[0].DATAMASK = dm;
      break;
    case en_stdio_portb:
      dm = VOR_GPIO->BANK[1].DATAMASK;
      VOR_GPIO->BANK[1].DATAMASK  = 0xff;       // Write DataMask
      VOR_GPIO->BANK[1].DATAOUT   = 0x80 | c;  // Pulse bit 7
      VOR_GPIO->BANK[1].DATAMASK = dm;
      break;
  }
  return c;
}

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
